@@ -1,3 +1,5 @@
+import logging
+import time
 from contextlib import suppress
 from enum import Enum
 from typing import Any, Dict, List, Optional
@@ -8,6 +10,7 @@ from pydantic import Field, SecretStr, create_model, root_validator
 from pydantic.types import constr
 
 from toucan_connectors.common import ConnectorStatus
+from toucan_connectors.connection_manager import ConnectionManager
 from toucan_connectors.sql_query_helper import SqlQueryHelper
 from toucan_connectors.toucan_connector import (
     DataSlice,
@@ -37,6 +40,14 @@ ORDERED_KEYS = [
     'region',
 ]
 
+logger = logging.getLogger(__name__)
+
+redshift_connection_manager = None
+if not redshift_connection_manager:
+    redshift_connection_manager = ConnectionManager(
+        name='redshift', timeout=25, wait=0.2, time_between_clean=10, time_keep_alive=60
+    )
+
 
 class AuthenticationMethod(str, Enum):
     DB_CREDENTIALS: str = 'db_credentials'
@@ -153,6 +164,10 @@ class RedshiftConnector(ToucanConnector):
             raise ValueError(AuthenticationMethodError.UNKNOWN.value)
         return values
 
+    @staticmethod
+    def get_redshift_connection_manager() -> ConnectionManager:
+        return redshift_connection_manager
+
     def _get_connection_params(self, database) -> Dict:
         con_params = dict(
             database=database,
@@ -182,14 +197,38 @@ class RedshiftConnector(ToucanConnector):
 
     def _get_connection(self, datasource) -> redshift_connector.Connection:
         """Establish a connection to an Amazon Redshift cluster."""
-        return redshift_connector.connect(
-            **self._get_connection_params(
-                database=datasource.database if datasource is not None else None,
+
+        def connect_function() -> redshift_connector.Connection:
+            return redshift_connector.connect(
+                **self._get_connection_params(
+                    database=datasource.database if datasource is not None else None,
+                )
             )
+
+        def alive_function(connection) -> bool:
+            logger.info(f'Alive Redshift connection: {connection}')
+            return self._is_alive
+
+        def close_function(connection) -> None:
+            logger.info('Close Redshift connection')
+            if not self._is_alive:
+                return connection.close()
+
+        connection: RedshiftConnector = redshift_connection_manager.get(
+            identifier=f'{self.get_identifier()}{datasource.database}{self.cluster_identifier}',
+            connect_method=connect_function,
+            alive_method=alive_function,
+            close_method=close_function,
+            save=True if datasource.database else False,
         )
+        connection.paramstyle = 'pyformat'
+        if self.connect_timeout is not None:
+            time.sleep(self.connect_timeout)
+            self._is_alive = False
+        return connection
 
     def _get_cursor(self, datasource) -> redshift_connector.Cursor:
-        return self._get_connection(datasource=datasource).cursor()
+        return self._get_connection(datasource=datasource).__enter__().cursor()
 
     def _retrieve_tables(self, datasource) -> List[str]:
         with self._get_connection(datasource=datasource) as conn:
@@ -277,30 +316,4 @@ class RedshiftConnector(ToucanConnector):
             self.check_port(self.host, self.port)
         except Exception as e:
             return ConnectorStatus(status=False, details=self._get_details(1, False), error=str(e))
-        # Check connection
-        try:
-            with self._get_connection(None):
-                return ConnectorStatus(status=True, details=self._get_details(2, True), error=None)
-        except redshift_connector.error.ProgrammingError as ex:
-            # Use to validate if the issue is "only" an issue with database (set after with datasource)
-            if (
-                f"'S': 'FATAL', 'C': '3D000', 'M': 'database \"{self.user}\" does not exist'"
-                in str(ex)
-            ):
-                return ConnectorStatus(
-                    status=True, details=self._get_details(2, True), error=str(ex)
-                )
-            # Same check for IAM mode
-            elif (
-                f"'S': 'FATAL', 'C': '3D000', 'M': 'database \"IAM:{self.db_user}\" does not exist'"
-                in str(ex)
-            ):
-                return ConnectorStatus(
-                    status=True, details=self._get_details(2, True), error=str(ex)
-                )
-            else:
-                return ConnectorStatus(
-                    status=False, details=self._get_details(2, False), error=str(ex)
-                )
-        except Exception as e:
-            return ConnectorStatus(status=False, details=self._get_details(2, False), error=str(e))
+        return ConnectorStatus(status=True, details=self._get_details(2, True), error=None)
